@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: axis.c,v 1.60.2.2 2007/03/22 04:25:41 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: axis.c,v 1.77.2.4 2009/11/04 16:11:25 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - axis.c */
@@ -600,6 +600,11 @@ make_tics(AXIS_INDEX axis, int guide)
     double xr, tic;
 
     xr = fabs(axis_array[axis].min - axis_array[axis].max);
+    if (xr == 0)
+	return 1;	/* Anything will do, since we'll never use it */
+    if (xr >= VERYLARGE)
+	int_error(NO_CARET,"%s axis range undefined or overflow",
+		axis_defaults[axis].name);
     tic = quantize_normal_tics(xr, guide);
     /* FIXME HBB 20010831: disabling this might allow short log axis
      * to receive better ticking... */
@@ -788,15 +793,19 @@ setup_tics(AXIS_INDEX axis, int max)
 	autoextend_min = autoextend_max = FALSE;
     }
 
-    /* BUGFIX HBB 20010831: for time/date axes, if an explicit
-     * stepsize was set, timelevel[axis] wasn't defined, leading to
-     * strange misbehaviours of minor tics on time axes. This would
-     * usually be used to redefine the 'tic' interval, but as a side
-     * effect, it defines timelevel[axis]. */
-    /* HBB 20011204: moved this up --- round_outward() needs
-     * timelevel[axis], too */
-    if (this->is_timedata && ticdef->type == TIC_SERIES)
-	quantize_time_tics(axis, tic, fabs(this->max - this->min), 20);
+    /* If an explicit stepsize was set, timelevel[axis] wasn't defined,
+     * leading to strange misbehaviours of minor tics on time axes.
+     * We used to call quantize_time_tics, but that also caused strangeness.
+     */
+    if (this->is_timedata && ticdef->type == TIC_SERIES) {
+	if      (tic >= 365*24*60*60.) timelevel[axis] = TIMELEVEL_YEARS;
+	else if (tic >=  28*24*60*60.) timelevel[axis] = TIMELEVEL_MONTHS;
+	else if (tic >=   7*24*60*60.) timelevel[axis] = TIMELEVEL_WEEKS;
+	else if (tic >=     24*60*60.) timelevel[axis] = TIMELEVEL_DAYS;
+	else if (tic >=        60*60.) timelevel[axis] = TIMELEVEL_HOURS;
+	else if (tic >=           60.) timelevel[axis] = TIMELEVEL_MINUTES;
+	else                           timelevel[axis] = TIMELEVEL_SECONDS;
+    }
 
     if (autoextend_min)
 	this->min = round_outward(axis, ! (this->min < this->max), this->min);
@@ -866,12 +875,14 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	    if (!inrange(internal, internal_min, internal_max))
 		continue;
 
-	    if (axis_array[axis].is_timedata)
+	    if (mark->level < 0) /* label read from data file */
+		strncpy(label, mark->label, sizeof(label));
+	    else if (axis_array[axis].is_timedata)
 		gstrftime(label, 24, mark->label ? mark->label : ticfmt[axis], mark->position);
 	    else
 		gprintf(label, sizeof(label), mark->label ? mark->label : ticfmt[axis], log10_base, mark->position);
 	    /* use NULL instead of label for minitic */
-	    (*callback) (axis, internal, mark->level?NULL:label, mark->level?mgrd:lgrd);
+	    (*callback) (axis, internal, (mark->level>0)?NULL:label, (mark->level>0)?mgrd:lgrd, NULL);
 	}
 	if (def->type == TIC_USER)
 	    return;
@@ -897,7 +908,6 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	double lmin = axis_array[axis].min, lmax = axis_array[axis].max;
 	double internal_min, internal_max;	/* to allow for rounding errors */
 	double ministart = 0, ministep = 1, miniend = 1;	/* internal or user - depends on step */
-	int anyticput = 0;	/* for detection of infinite loop */
 
 	/* gprintf uses log10() of base - log_base_array is log() */
 	double log10_base = axis_array[axis].log ? log10(axis_array[axis].base) : 1.0;
@@ -970,15 +980,13 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	step = fabs(step);
 	/* }}} */
 
-	if (minitics) {
+	if (minitics && axis_array[axis].miniticscale != 0) {
 	    /* {{{  figure out ministart, ministep, miniend */
 	    if (minitics == MINI_USER) {
 		/* they have said what they want */
 		if (minifreq <= 0)
 		    minitics = 0;	/* not much else we can do */
  		else if (axis_array[axis].log) {
-		    /* Sep 2005 - This case has been commented out since v3.7 */
-		    /* but in fact it seems correct, and fixes but #1223149 */
  		    ministart = ministep = step / minifreq * axis_array[axis].base;
  		    miniend = step * axis_array[axis].base;
  		} else {
@@ -989,7 +997,10 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 		if (step > 1.5) {	/* beware rounding errors */
 		    /* {{{  10,100,1000 case */
 		    /* no more than five minitics */
-		    ministart = ministep = (int) (0.2 * step);
+		    if (step < 65535) /* should be MAXINT */
+			ministart = ministep = (int)(0.2 * step);
+		    else
+			ministart = ministep = 0.2 * step;
 		    if (ministep < 1)
 			ministart = ministep = 1;
 		    miniend = step;
@@ -1044,29 +1055,36 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	    return;		/* just quietly ignore them ! */
 	/* }}} */
 
-	/* FIXME HBB 20010121: keeping adding 'step' to 'tic' is
-	 * begging for rounding errors to strike us. */
-	/* HBB 20010410: ... and strike they did :-( */
+	/* This protects against user error, not precision errors */
+	if ( (internal_max-internal_min)/step > term->xmax) {
+	    int_warn(NO_CARET,"Too many axis ticks requested (>%.0g)",
+		(internal_max-internal_min)/step);
+	    return;
+	}
+
+	/* This protects against infinite loops if the separation between   */
+	/* two ticks is less than the precision of the control variables.   */
+	/* The for(...) loop here must be identical to the true loop below. */
+	if (1) /* (some-test-for-range-and-or-step-size) */ {
+	    int anyticput = 0;
+	    double previous_tic = 0;
+
+	    for (tic = start; tic <= end; tic += step) {
+		/* EAM Oct 2008: Previous code (2001) checked only the start and end
+		 * points, but rounding error can strike at any point in the range.
+		 */
+		if (anyticput == 0)
+		    anyticput = 1;
+		else if (fabs(tic - previous_tic) < (step/4.)) {
+		    step = end - start;
+		    int_warn(NO_CARET, "tick interval too small for machine precision");
+		    break;
+		}
+		previous_tic = tic;
+	    }
+	}
+
 	for (tic = start; tic <= end; tic += step) {
-	    if (anyticput == 2)	/* See below... */
-		break;
-	    /* HBB 20010121: Previous code checked absolute value
-	     * DBL_EPSILON against tic distance. That's numerical
-	     * nonsense. It essentially disallowed series ticmarks for
-	     * all axes shorter than DBL_EPSILON in absolute figures.
-	     * */
-	    if (anyticput) {
-	      if (NearlyEqual(tic, start, step)) {
-		/* step is too small.. */
-		anyticput = 2;	/* Don't try again. */
-		tic = end;	/* Put end tic. */
-	      } else if (NearlyEqual(tic, end, step)) {
-		/* HBB 20010410: protect against cancellation at upper
-		 * end, too (can happen if |step/end| <= ~DBL_EPS). */
-		anyticput = 2;
-	      }
-	    } else
-		anyticput = 1;
 
 	    /* {{{  calc internal and user co-ords */
 	    if (!axis_array[axis].log) {
@@ -1083,23 +1101,22 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	    if (internal > internal_max)
 		break;		/* gone too far - end of series = VERYLARGE perhaps */
 	    if (internal >= internal_min) {
-#if 0 /* maybe minitics!!!. user series starts below min ? */
-		continue;
-#endif
 		/* {{{  draw tick via callback */
 		switch (def->type) {
 		case TIC_DAY:{
 			int d = (long) floor(user + 0.5) % 7;
 			if (d < 0)
 			    d += 7;
-			(*callback) (axis, internal, abbrev_day_names[d], lgrd);
+			(*callback) (axis, internal, abbrev_day_names[d], lgrd,
+					def->def.user);
 			break;
 		    }
 		case TIC_MONTH:{
 			int m = (long) floor(user - 1) % 12;
 			if (m < 0)
 			    m += 12;
-			(*callback) (axis, internal, abbrev_month_names[m], lgrd);
+			(*callback) (axis, internal, abbrev_month_names[m], lgrd,
+					def->def.user);
 			break;
 		    }
 		default:{	/* comp or series */
@@ -1109,28 +1126,26 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 			    gstrftime(label, 24, ticfmt[axis], (double) user);
 			} else if (polar) {
 			    /* if rmin is set, we stored internally with r-rmin */
-			    /* HBB 990327: reverted to 'pre-Igor' version... */
-#if 1				/* Igor's polar-grid patch */
 			    double r = fabs(user) +
 				((axis_array[R_AXIS].autoscale & AUTOSCALE_MIN)
 				 ? 0 : axis_array[R_AXIS].min);
-#else
-			    /* Igor removed fabs to allow -ve labels */
-			    double r = user +
-				((axis_array[R_AXIS].autoscale & AUTOSCALE_MIN)
-				 ? 0 : axis_array[R_AXIS].min);
-#endif
 			    gprintf(label, sizeof(label), ticfmt[axis], log10_base, r);
 			} else {
 			    gprintf(label, sizeof(label), ticfmt[axis], log10_base, user);
 			}
-			(*callback) (axis, internal, label, lgrd);
+
+			/* Range-limited tic placement */
+			if (def->rangelimited
+			&&  !inrange(internal,axis_array[axis].data_min,axis_array[axis].data_max))
+			    continue;
+
+			(*callback) (axis, internal, label, lgrd, def->def.user);
 		    }
 		}
 		/* }}} */
 
 	    }
-	    if (minitics) {
+	    if (minitics && axis_array[axis].miniticscale != 0) {
 		/* {{{  process minitics */
 		double mplace, mtic;
 		for (mplace = ministart; mplace < miniend; mplace += ministep) {
@@ -1144,7 +1159,7 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 			       : mplace);
 		    if (inrange(mtic, internal_min, internal_max)
 			&& inrange(mtic, start - step * SIGNIF, end + step * SIGNIF))
-			(*callback) (axis, mtic, NULL, mgrd);
+			(*callback) (axis, mtic, NULL, mgrd, NULL);
 		}
 		/* }}} */
 	    }
@@ -1417,16 +1432,13 @@ load_range(AXIS_INDEX axis, double *a, double *b, t_autoscale autoscale)
 
     /* HBB 20030127: If range input backwards, automatically turn on
        the "reverse" option, too. */
-    /* HBB 20040315: ... and clear it automatically if a fixed range
-     * was given the "right" way round! */
     if ((autoscale & AUTOSCALE_BOTH) == AUTOSCALE_NONE) {
       if (*b < *a) {
 	double temp = *a;
 
 	*a = *b; *b = temp;
 	axis_array[axis].range_flags |= RANGE_REVERSE;
-      } else 
-	axis_array[axis].range_flags &= ~RANGE_REVERSE;
+      }
     }
 
     return (autoscale);
@@ -1440,7 +1452,9 @@ load_range(AXIS_INDEX axis, double *a, double *b, t_autoscale autoscale)
  */
 
 void
-widest_tic_callback(AXIS_INDEX axis, double place, char *text, struct lp_style_type grid)
+widest_tic_callback(AXIS_INDEX axis, double place, char *text,
+    struct lp_style_type grid,
+    struct ticmark *userlabels)
 {
     (void) axis;		/* avoid "unused parameter" warnings */
     (void) place;
@@ -1508,10 +1522,6 @@ some_grid_selected()
 int
 set_cbminmax()
 {
-#if 0
-    printf("ENTER set_cbminmax:  Z_AXIS.min=%g\t Z_AXIS.max=%g\n",Z_AXIS.min,Z_AXIS.max);
-    printf("ENTER set_cbminmax: CB_AXIS.min=%g\tCB_AXIS.max=%g\n",CB_AXIS.min,CB_AXIS.max);
-#endif
     if (CB_AXIS.set_autoscale & AUTOSCALE_MIN) {
 	/* -VERYLARGE according to AXIS_INI3D */
 	if (CB_AXIS.min >= VERYLARGE)
@@ -1526,22 +1536,13 @@ set_cbminmax()
     }
     CB_AXIS.max = axis_log_value_checked(COLOR_AXIS, CB_AXIS.max, "color axis");
 
-#if 0
-    if (CB_AXIS.min == CB_AXIS.max) {
-	int_error(NO_CARET, "cannot display empty color axis range");
-	return 0;
-    }
-#endif
     if (CB_AXIS.min > CB_AXIS.max) {
 	/* exchange min and max values */
 	double tmp = CB_AXIS.max;
 	CB_AXIS.max = CB_AXIS.min;
 	CB_AXIS.min = tmp;
     }
-#if 0
-    printf("EXIT  set_cbminmax:  Z_AXIS.min=%g\t Z_AXIS.max=%g\n",Z_AXIS.min,Z_AXIS.max);
-    printf("EXIT  set_cbminmax: CB_AXIS.min=%g\tCB_AXIS.max=%g\n",CB_AXIS.min,CB_AXIS.max);
-#endif
+
     return 1;
 }
 
@@ -1594,6 +1595,8 @@ get_position_default(struct position *pos, enum position_type default_type)
     int axes;
     enum position_type type = default_type;
 
+    memset(pos, 0, sizeof(struct position));
+
     get_position_type(&type, &axes);
     pos->scalex = type;
     GET_NUMBER_OR_TIME(pos->x, axes, FIRST_X_AXIS);
@@ -1611,7 +1614,8 @@ get_position_default(struct position *pos, enum position_type default_type)
     /* z is not really allowed for a screen co-ordinate, but keep it simple ! */
     if (equals(c_token, ",")
        /* Partial fix for ambiguous syntax when trailing comma ends a plot command */
-	&& !(isstringvalue(c_token+1))
+	&& !(isstringvalue(c_token+1)) && !(almost_equals(c_token+1,"newhist$ogram"))
+	&& !(almost_equals(c_token+1,"for"))
        ) {
 	++c_token;
 	get_position_type(&type, &axes);
@@ -1627,8 +1631,8 @@ get_position_default(struct position *pos, enum position_type default_type)
  * Add a single tic mark, with label, to the list for this axis.
  * To avoid duplications and overprints, sort the list and allow
  * only one label per position.
- * EAM - called from set.c during `set xtics`
- *       called from datafile.c during `plot using ::xtic()`
+ * EAM - called from set.c during `set xtics` (level = 0 or 1)
+ *       called from datafile.c during `plot using ::xtic()` (level = -1)
  */
 void
 add_tic_user(AXIS_INDEX axis, char *label, double position, int level)
@@ -1636,6 +1640,9 @@ add_tic_user(AXIS_INDEX axis, char *label, double position, int level)
     struct ticmark *tic, *newtic;
     struct ticmark listhead;
 
+    if (!label && level < 0)
+	return;
+
     /* Mark this axis as user-generated ticmarks only, unless the */
     /* mix flag indicates that both user- and auto- tics are OK.  */
     if (!axis_array[axis].ticdef.def.mix)
@@ -1660,7 +1667,7 @@ add_tic_user(AXIS_INDEX axis, char *label, double position, int level)
     } else {
 	/* The new tic must duplicate position of tic->next */
 	if (position != tic->next->position)
-	    fprintf(stderr,"add_tic_user: list sort error\n");
+	    int_warn(NO_CARET, "add_tic_user: list sort error");
 	newtic = tic->next;
 	/* Don't over-write a major tic with a minor tic */
 	if (newtic->level < level)
